@@ -22,12 +22,13 @@
 
 /* TODO
  *
- * Add getty support and UUCP locking
  * Add some type of security
  */
 
 #include <stdio.h>
 #include <signal.h>
+#include <stdlib.h>
+#include <string.h>
 #include <syslog.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -39,9 +40,15 @@
 #include "dataxfer.h"
 
 static char *config_file = "/etc/ser2net.conf";
-static int config_port = -1;
+static char *config_port = NULL;
 static int detach = 1;
 static int debug = 0;
+#ifdef USE_UUCP_LOCKING
+int uucp_locking_enabled = 1;
+#endif
+int cisco_ios_baud_rates = 0;
+
+selector_t *ser2net_sel;
 
 static char *help_string =
 "%s: Valid parameters are:\n"
@@ -49,8 +56,19 @@ static char *help_string =
 "  -p <controller port> - Start a controller session on the given TCP port\n"
 "  -n - Don't detach from the controlling terminal\n"
 "  -d - Don't detach and send debug I/O to standard output\n"
+#ifdef USE_UUCP_LOCKING
+"  -u - Disable UUCP locking\n"
+#endif
+"  -b - Do CISCO IOS baud-rate negotiation, instead of RFC2217\n"
 "  -v - print the program's version and exit\n";
 
+void
+reread_config(void)
+{
+    syslog(LOG_INFO, "Got SIGHUP, re-reading configuration");
+    readconfig(config_file);
+}
+
 void
 arg_error(char *name)
 {
@@ -62,6 +80,7 @@ int
 main(int argc, char *argv[])
 {
     int i;
+    int err;
 
     for (i=1; i<argc; i++) {
 	if ((argv[i][0] != '-') || (strlen(argv[i]) != 2)) {
@@ -79,6 +98,10 @@ main(int argc, char *argv[])
 	    debug = 1;
 	    break;
 
+	case 'b':
+	    cisco_ios_baud_rates = 1;
+	    break;
+
 	case 'c':
 	    /* Get a config file. */
 	    i++;
@@ -96,13 +119,15 @@ main(int argc, char *argv[])
 		fprintf(stderr, "No control port specified with -p\n");
 		arg_error(argv[0]);
 	    }
-	    config_port = scan_int(argv[i]);
-	    if (config_port == -1) {
-		fprintf(stderr, "Invalid control port specified with -p\n");
-		arg_error(argv[0]);
-	    }
+	    config_port = argv[i];
 	    break;
 
+#ifdef USE_UUCP_LOCKING
+	case 'u':
+	    uucp_locking_enabled = 0;
+	    break;
+#endif
+
 	case 'v':
 	    printf("%s version %s\n", argv[0], VERSION);
 	    exit(0);
@@ -113,10 +138,19 @@ main(int argc, char *argv[])
 	}
     }
 
-    selector_init();
-    dataxfer_init();
-    if (config_port != -1) {
-	controller_init(config_port);
+    err = sel_alloc_selector(&ser2net_sel);
+    if (err) {
+	fprintf(stderr,
+		"Could not initialize ser2net selector: '%s'\n",
+		strerror(err));
+	return -1;
+    }
+    setup_sighup();
+    if (config_port != NULL) {
+	if (controller_init(config_port) == -1) {
+	    fprintf(stderr, "Invalid control port specified with -p\n");
+	    arg_error(argv[0]);
+	}
     }
 
     if (readconfig(config_file) == -1) {
@@ -134,11 +168,13 @@ main(int argc, char *argv[])
 	    syslog(LOG_ERR, "Error forking first fork");
 	    exit(1);
 	} else {
+	    /* setsid() is necessary if we really want to demonize */
+	    setsid();
 	    /* Second fork to really deamonize me. */
 	    if ((pid = fork()) > 0) {
 		exit(0);
 	    } else if (pid < 0) {
-		syslog(LOG_ERR, "Error forking first fork");
+		syslog(LOG_ERR, "Error forking second fork");
 		exit(1);
 	    }
 	}
@@ -155,7 +191,9 @@ main(int argc, char *argv[])
     /* Ignore SIGPIPEs so they don't kill us. */
     signal(SIGPIPE, SIG_IGN);
 
-    select_loop();
+    set_sighup_handler(reread_config);
+
+    sel_select_loop(ser2net_sel);
 
     return 0;
 }
